--[[
	Description: Titan plug-in to display Herbalism Data
	Author: Linuz
--]]

local Addon = "DataMiner";						  -- 00000 Bitfield viewmode
local collect = false;	--enable dm				  -- ║║║║╚═ aMapId
local logg = true;								  -- ║║║╚══ aSeg
local logsize = 50;	    --max lines in log		  -- ║║╚═══ aDay
local mapId;									  -- ║╚════ aName
local viewmode = 7;     --what to show in tooltip -- ╚═════ Cumulative list
local search;           --for viewmode 8
dm_herbs, dm_ore = {}, {};
dm_log = {};

-- Library references
local L = LibStub("AceLocale-3.0"):GetLocale(Addon, true);
local AceGUI = LibStub("AceGUI-3.0");

function TitanPanelRightClickMenu_PrepareDMMenu()
	--f = CreateFrame("EditBox", nil, self)
	--f:SetWidth(40)
	--f:SetHeight(20)
	--f:SetPoint("TOPLEFT", self, "TOPLEFT", 5, -10)
	--f:SetFontObject("GameFontHighlight")
	--local f = AceGUI:Create("Frame")
	--f:SetCallback("OnClose",function(widget) AceGUI:Release(widget) end)
	--f:SetStatusText("Status Bar")
	--f:SetLayout("Flow")
	--local btn = AceGUI:Create("EditBox")
	--btn:SetWidth(170)
	--btn:SetText("Button !")
	--btn:SetCallback("OnClick", function() print("Click!") end)
	-- Add the button to the container
	--AddChild(btn)
	
	if ( UIDROPDOWNMENU_MENU_LEVEL == 1 ) then
		TitanPanelRightClickMenu_AddTitle("DataMiner");
		local info = UIDropDownMenu_CreateInfo();
		info.text = L["Toggle_loging"];
		info.func = function() DM_Toggle_Log() end;
		info.checked = logg;
		info.keepShownOnClick = true;
		info.isNotRadio = true;    -- as checkmark
		info.tooltipTitle = "Enable/Disable loging"
		UIDropDownMenu_AddButton(info, 1);
	
		info = UIDropDownMenu_CreateInfo();
		info.text = "Logsize";
		info.hasArrow = true;    	-- as submenue
		info.notCheckable = true; 	-- text only
		info.notClickable = true;
		info.keepShownOnClick = true;
		UIDropDownMenu_AddButton(info, 1);
		
		info = UIDropDownMenu_CreateInfo();
		info.text = L["Toggle_datamining"];
		info.func = function() ToggleDataMining(self) end;
		info.checked = collect;
		if ((not HasProfession(PROF_HERBALISM,false)) and
			(not HasProfession(PROF_MINING,false))) then info.disabled = true end;
		info.keepShownOnClick = true;
		info.isNotRadio = true;    	-- as checkmark
		UIDropDownMenu_AddButton(info, 1);
	end	
	
	if ( UIDROPDOWNMENU_MENU_LEVEL == 2 ) then
		info = UIDropDownMenu_CreateInfo();
		info.text = "50";
		info.func = function() logsize = 50 end;
		info.notCheckable = true; 	-- text only
		info.keepShownOnClick = true;
		UIDropDownMenu_AddButton(info, 2);
		
		info = UIDropDownMenu_CreateInfo();
		info.text = "100";
		info.func = function() logsize = 100 end;
		info.notCheckable = true; 	-- text only
		info.keepShownOnClick = true;
		UIDropDownMenu_AddButton(info, 2);
	end
end

function DM_Create_Tip(dm_base)
	local iDay = date("%A");
	local iSeg = TimeSegment(8);
	local maxlootrate = 0;
	local iMapId;
	
	for idx, line in ipairs(dm_base) do
		if ((line.aDay == iDay) and
			(line.aSeg == iSeg) and
			(line.aLoot > 10)) then --and (line.aCount > 20)
			if (maxlootrate < (line.aCount / line.aLoot)) then
				maxlootrate = line.aCount / line.aLoot;
				iMapId = line.aMapId;
			end
		end
	end
	if (maxlootrate > 1.5) then
		local res;
		if (dm_base == dm_herbs) then res = "herbs" end;
		if (dm_base == dm_ore) then res = "ore" end;
		print("Go to "..GetMapNameByID(iMapId).." for "..res..
			  " because lootrate there is at "..string.format("%.2f", maxlootrate).." now!");
	end
	--DEFAULT_CHAT_FRAME:AddMessage(GREEN_FONT_COLOR_CODE.."Huhu"..FONT_COLOR_CODE_CLOSE);
end

function ToggleDataMining(self)
	local hasherbalism, hasmining = HasProfession(PROF_HERBALISM, false), HasProfession(PROF_MINING, false);
	if (not hasherbalism and not hasmining) then return end;
	do
		local states = 1;
		if (hasherbalism) then states = inc(states) end
		if (hasmining) then states = inc(states) end
		collect = ToggleX(states, collect);
	end
	viewmode = 7; search = nil; --when we switch datamining, we reset the viewmode to default
	if (collect) then
		self:RegisterEvent("LOOT_OPENED");
		self:RegisterEvent("ZONE_CHANGED_NEW_AREA");
		--mapid's are unique numeric identifiers for zone maps.
		--They can be obtained by calling the GetCurrentMapAreaID.
		--These value can be passed to the GetMapNameByID API to identify zones.
		SetMapToCurrentZone();
		mapId = GetCurrentMapAreaID();
		DM_Log_Add(L["DataMining_is_now"].." "..L["on"]);
		if (hasherbalism) then DM_Create_Tip(dm_herbs) end
		if (hasmining) then DM_Create_Tip(dm_ore) end
	else
		DM_Log_Add(L["DataMining_is_now"].." "..L["off"]);
		self:UnregisterEvent("ZONE_CHANGED_NEW_AREA");
		self:UnregisterEvent("LOOT_OPENED");
	end
end

function DM_Array_getIndex(dm_base, iId, iMapId, iDay, iSeg)
	for i, line in ipairs(dm_base) do
		if ((line.aItemID == iId) and
		    (line.aMapId == iMapId) and
			(line.aDay == iDay) and
			(line.aSeg == iSeg)) then
			return i;
		end
	end
	return 0;
end

function DM_Data_Mining()
	local idx;
	local itemIcon, itemName, itemCount, itemQuality;
	local itemLink, itemId;
	local iDay = date("%A");
	local iSeg = TimeSegment(8);
	local dm_base;
	
	-- Which table should we use
	if ( collect == 1 ) then
		if (HasProfession(PROF_HERBALISM, false)) then dm_base = dm_herbs
		elseif (HasProfession(PROF_MINING, false)) then dm_base = dm_ore end
	elseif ( collect == 2 )	then
		if (HasProfession(PROF_MINING, false)) then dm_base = dm_ore end
	end
	
	for i=1, GetNumLootItems(), 1 do
		itemLink = GetLootSlotLink(i);
		itemId = itemLink and tonumber(strmatch(itemLink, "item:(%d+)"));
		itemIcon, itemName, itemCount, itemQuality = GetLootSlotInfo(i);
		
		if ((IsHerb(itemId)) and (dm_base == dm_herbs) or (IsOre(itemId)) and (dm_base == dm_ore)) then
			idx = DM_Array_getIndex(dm_base, itemId, mapId, iDay, iSeg);    --Look if this item is already there
			if (idx>0) then                                         		 --Already there
				DM_Log_Add(L["Updating"].." "..itemName.." ID = "..itemId.." Zone = "..GetMapNameByID(mapId));
				PlaySoundFile("Interface\\AddOns\\TitanDataMiner\\sounds\\update.mp3");
				dm_base[idx].aCount = dm_base[idx].aCount + itemCount;
				dm_base[idx].aLoot = inc(dm_base[idx].aLoot);
			else                                                    		 --This is a new entry
				DM_Log_Add(L["Adding"].." "..itemName.." ID = "..itemId.." Zone = "..GetMapNameByID(mapId));
				PlaySoundFile("Interface\\AddOns\\TitanDataMiner\\sounds\\add.mp3");
				table.insert (dm_base, {aItemID = itemId, aMapId = mapId, aDay = iDay, aSeg = iSeg, aCount = itemCount, aLoot = 1});
			end
		else
			--DM_Log_Add(L["Canceling"].." "..itemName.." ID = "..itemId.." Zone = "..GetMapNameByID(mapId));
			--PlaySoundFile("Interface\\AddOns\\TitanDataMiner\\sounds\\cancel.mp3");
		end	
	end
end

-------------
-- Loging --
-------------

-- Add a new line to log
-- If the maximum capacity is reached, then delete old entrys
function DM_Log_Add(iLog)
	--print(iLog);
	if (not logg) then return; end
	if ( #dm_log >= logsize ) then
		repeat
			table.remove(dm_log, 1);         --Del oldest msg until size is valid
		until #dm_log <= logsize;
	end
	table.insert (dm_log, {aLog = iLog});
end

-- Enable/Disable the Loging feature
function DM_Toggle_Log()
	if (logg) then
		logg = false
		DM_Log_Add(L["Loging_is_now"].." "..L["off"]);
	else
		logg = true
		DM_Log_Add(L["Loging_is_now"].." "..L["on"]);
	end
end

-------------------
-- Chat Commands --
-------------------

-- Print the Herbalism Table to chat
-- only for chat "/dm table herbs"
-- used from foreach()
function DM_Herbs_Print(idx)
	print(idx, dm_herbs[idx].aDay,
	      dm_herbs[idx].aSeg,
		  dm_herbs[idx].aMapId,
		  dm_herbs[idx].aItemID,
		  " x"..dm_herbs[idx].aCount,
		  string.format("%.2f", dm_herbs[idx].aCount/dm_herbs[idx].aLoot));
end

-- Print the Ore Table to chat
-- only for chat "/dm table ore"
-- used from foreach()
function DM_Ore_Print(idx)
	print(idx, dm_ore[idx].aDay,
	      dm_ore[idx].aSeg,
		  dm_ore[idx].aMapId,
		  dm_ore[idx].aItemID,
		  " x"..dm_ore[idx].aCount,
		  string.format("%.2f", dm_ore[idx].aCount/dm_ore[idx].aLoot));
end

-- only for chat "/dm log"
-- used from foreach
function DM_Log_Print(idx)
	print(idx.." "..dm_log[idx].aLog);
end

-- Builds a parameter list from chat input
-- Parameter: str     e.g /dm view item "my cool item"
-- Return: argc, args[argc]
local function DM_Slash_Parser(str)
	local param, first, second;
	local q_open = false;
	local argc = 0;
	local args = {};
		
	if ( str == "" ) then return argc, args end
	-- "([^\t\n]*) \t? ([^\t\n]*)\n"
	for param in string.gmatch(str, '[%w\128-\244\"]+') do --we need %w for /dm table remove <#>
		first = string.find(param, "\"");
		second = string.find(param, "\"", 2);
		if (first == second) then first = false end
		if (first and second) then
			param = string.sub(param, 2, second-first)     -- msg is quoted
			argc = inc(argc); args[argc] = param;
		elseif (first and (not second)) then               -- msg is quoted beginning part
			param = string.sub(param, 2)
			argc = inc(argc); args[argc] = param;
			q_open = true;                                 -- q_open: needed for quoted 3 or more part msg
		elseif ((not first) and second) then               -- msg is quoted ending part
			param = string.sub(param, 1, dec(second));
			args[argc] = args[argc].." "..param;
			q_open = false;
		else
			if (q_open) then                            -- msg is quoted middle part
				args[argc] = args[argc].." "..param;
			else
				argc = inc(argc); args[argc] = param;   -- msg is not quoted
			end
		end
	end
	return argc, args
end

function DM_SlashHandler(msg)
	local args, argc = {};
	argc, args = DM_Slash_Parser(msg);
	
	if(args[1]=="test") then
		TitanPlugins["DM"].tooltipTitle = "Test"
	end
	
	if ((args[1] == "help") or (argc == 0)) then
		print(L["Available_commands"]..":");
		print("/dm "..L["or"].." /dm help                     - "..L["This_message"]);
		print("/dm toggle          ".."                      - "..L["Toggle_datamining"]);
		print("/dm table herbs|ore ".."                   - "..L["Show_table"]);
		print("/dm table herbs|ore remove <n>".."  - "..L["Removes_index_n_from_table"]);
		print("/dm time            ".."                      - "..L["Show_current_time_segment"]);
		print("/dm log             ".."                       - "..L["Show_log"]);
		print("/dm log erase       ".."                    - "..L["Delete_log"]);
		print("/dm log toggle      ".."                    - "..L["Toggle_loging"]);
		print("/dm view default    ".."                   - "..L["Default_view"]);
		print("/dm view item|zone <name>".."          - "..L["Item_view"]);
	elseif (args[1] == "view") then
		if (args[2] == "default") then
			viewmode = 7; search = nil;        --views current zone,day,time
		elseif ((args[2] == "item") and (argc == 3)) then
			viewmode = 8; search = args[3];    -- view name
		elseif ((args[2] == "zone") and (argc == 3)) then
			viewmode = 17; search = args[3];   -- views cumulative zone
		end
	elseif (args[1] == "toggle") then
		ToggleDataMining(self);
	elseif (args[1] == "time") then
		print (L["Current_time_segment_is"].." "..TimeSegment(8));
	elseif (args[1] == "table") then
		if (args[2] == "herbs" ) then
			if (argc == 2) then
				if (table.getn(dm_herbs) > 0) then
					print(Addon.." "..L["Herbalism_Statistics"]..":");
					table.foreach(dm_herbs, DM_Herbs_Print);
				end
			elseif ((args[3] == "remove") and (argc == 4)) then
				table.remove(dm_herbs, tonumber(args[4]));
				DM_Log_Add(L["Removing_from_table_at_index"].." "..args[4]);
			end	
		elseif (args[2] == "ore" ) then
			if (argc == 2) then
				if (table.getn(dm_ore) > 0) then
					print(Addon.." "..L["Mining_Statistics"]..":");
					table.foreach(dm_ore, DM_Ore_Print);
				end
			elseif ((args[3] == "remove") and (argc == 4)) then
				table.remove(dm_ore, tonumber(args[4]));
				DM_Log_Add(L["Removing_from_table_at_index"].." "..args[4]);
			end	
		end
	elseif (args[1] == "log") then
		if ((argc == 1) or ((argc == 2) and (args[2] == "show"))) then
			if (table.getn(dm_log) > 0) then
				print("DataMiner log:");
				table.foreach(dm_log, DM_Log_Print);
			end	
		elseif (args[2] == "delete") then
			Table_reset(dm_log);
			DM_Log_Add(L["Clearing_log"]);
		elseif (args[2] == "toggle") then
			DM_Toggle_Log();
		end
	end
end

------------
-- Events --
------------

function DM_OnEvent(ev, arg)
	if (ev == "LOOT_OPENED") then
		DM_Data_Mining();
	elseif (ev == "ZONE_CHANGED_NEW_AREA") then
		SetMapToCurrentZone();
		mapId = GetCurrentMapAreaID();
		DM_Log_Add(L["Entering"].." "..GetMapNameByID(mapId));
	elseif (ev == "PLAYER_ENTERING_WORLD") then
		local version = tostring(GetAddOnMetadata("TitanDataMiner", "Version"));
		local author = GetAddOnMetadata("TitanDataMiner", "Author");
		DM_Log_Add(Addon.." v"..version.." "..L["loaded"]);
	else
		print("Unknown Event "..ev);
	end
	return;
end

function TitanPanelDMButton_OnClick(self, mousebutton)
	if (mousebutton == "LeftButton") then
		ToggleDataMining(self);
	end
	if (mousebutton == "RightButton") then
		TitanPanelRightClickMenu_PrepareDMMenu()
	end	
end

function TitanPanelDMButton_GetTooltipText()
	local function GetIdx(buff, id) -- for viewmode 17
		for idx, line in ipairs(buff) do
			if (line.aItemID == id) then
				return idx;
			end
		end
		return 0;
	end
	--local function GetSpaces(num)
		--return string.rep (" ", num);
	--end
	
	-- Which table should we use
	local dm_base;
	if ( not collect ) then
		return L["Currently_mining_data_is"].." "..GetHexColoredText(L["off"], COL_GREEN2);
	elseif ( collect == 1 ) then
		if (HasProfession(PROF_HERBALISM, false)) then dm_base = dm_herbs
		elseif (HasProfession(PROF_MINING, false)) then dm_base = dm_ore end
	elseif ( collect == 2 )	then
		if (HasProfession(PROF_MINING, false)) then dm_base = dm_ore end
	end
	if (dm_base == nil) then
		return L["This_character_has_no_supported_profession"];
	end
	
	local dm_buff = {};
	local iDay = date("%A");
	local iSeg = TimeSegment(8);
	local iMapId = mapId;
	local out = "";
	local spacer = "-------------------------------------------------------\n"
	local header;
			
	if (viewmode == 7) then -- view name count loots rate
		header = spacer
				 ..GetHexColoredText(L["Item"],COL_YELLOW).."\t"
				 ..GetHexColoredText(L["Pieces"].."  ",COL_GREEN)
				 ..GetHexColoredText(L["Places"].."  ",COL_WHITE)
				 ..GetHexColoredText(L["Rate"],COL_ORANGE)
				 .."\n"..spacer;
		for idx, line in ipairs(dm_base) do
			if ((line.aMapId == iMapId) and (line.aSeg == iSeg) and (line.aDay == iDay)) then
				table.insert (dm_buff, dm_base[idx]);
			end
		end
		table.sort(dm_buff, function(a, b) return (a.aCount/a.aLoot) > (b.aCount/b.aLoot) end);
		for idx, line in ipairs(dm_buff) do
			out = out..GetHexColoredText(GetItemInfo(dm_buff[idx].aItemID), COL_YELLOW).."\t"..
		               GetHexColoredText(dm_buff[idx].aCount, COL_GREEN).."  "..
					   --GetSpaces((string.len(L["Pieces"]) - string.len(tostring(dm_herbs[idx].aCount))))..
					   GetHexColoredText(dm_buff[idx].aLoot, COL_WHITE).."  "..
					   --GetSpaces((string.len(L["Places"]) - string.len(tostring(dm_herbs[idx].aLoot))))..
					   GetHexColoredText(format("%.2f",dm_buff[idx].aCount / dm_buff[idx].aLoot), COL_ORANGE).."\n"
		end
		TitanPlugins["DM"].tooltipTitle = L["Current Zone, Weekday, Time"];
	elseif (viewmode == 8) then -- view day time zone rate
		header = spacer
				..GetHexColoredText(L["A_Time"].."  ",COL_GREEN)
				..GetHexColoredText(L["Weekday"],COL_YELLOW).."\t"
				..GetHexColoredText(L["Area"].."  ",COL_WHITE)
				..GetHexColoredText(L["Rate"],COL_ORANGE)
				.."\n"..spacer;
		for idx, line in ipairs(dm_base) do
			-- If not in cash GetItemInfo returns nil, in this
			-- case we have no chance to get the name!!
			if (GetItemInfo(dm_base[idx].aItemID)) then
				if (string.lower(GetItemInfo(dm_base[idx].aItemID)) == search) then
					table.insert (dm_buff, dm_base[idx]);
				end
			end
		end
		table.sort(dm_buff, function(a, b) return (a.aCount/a.aLoot) > (b.aCount/b.aLoot) end);
		for idx, line in ipairs(dm_buff) do
			out = out..GetHexColoredText(dm_buff[idx].aSeg, COL_GREEN).."  "..
					   GetHexColoredText(dm_buff[idx].aDay, COL_YELLOW).."\t"..
					   GetHexColoredText(GetMapNameByID(dm_buff[idx].aMapId), COL_WHITE).."  "..
					   GetHexColoredText(format("%.2f",dm_buff[idx].aCount / dm_buff[idx].aLoot), COL_ORANGE).."\n"
		end
		TitanPlugins["DM"].tooltipTitle = L["For Item:"].." "..search;
	elseif (viewmode == 17) then  -- Cumulative Zone View
		local idx;
		header = spacer
				 ..GetHexColoredText(L["Item"],COL_YELLOW).."\t"
				 ..GetHexColoredText(L["Pieces"].."  ",COL_GREEN)
				 ..GetHexColoredText(L["Places"].."  ",COL_WHITE)
				 ..GetHexColoredText(L["Rate"],COL_ORANGE)
				 .."\n"..spacer;
		-- We copy/alter the needed lines to dm_buff
		for i, line in ipairs(dm_base) do
			-- If not in cash GetItemInfo returns nil, in this
			-- case we have no chance to get the name!!
			if (GetItemInfo(line.aItemID)) then
				if (string.lower(GetMapNameByID(line.aMapId)) == search) then
					idx = GetIdx( dm_buff, line.aItemID );      -- Look if this item is already there
					if (idx>0) then   									     -- Already there
						-- We want to cumulate the items
						dm_buff[idx].aCount = dm_buff[idx].aCount + line.aCount;
						dm_buff[idx].aLoot = dm_buff[idx].aLoot + line.aLoot;
					else                                                     -- This is a new entry
						table.insert (dm_buff, line);
					end
				end
			end
		end
		table.sort(dm_buff, function(a, b) return (a.aCount/a.aLoot) > (b.aCount/b.aLoot) end);
		for idx, line in ipairs(dm_buff) do
			out = out..GetHexColoredText(GetItemInfo(dm_buff[idx].aItemID), COL_YELLOW).."\t"..
		               GetHexColoredText(dm_buff[idx].aCount, COL_GREEN).."  "..
					   --GetSpaces((string.len(L["Pieces"]) - string.len(tostring(dm_herbs[idx].aCount))))..
					   GetHexColoredText(dm_buff[idx].aLoot, COL_WHITE).."  "..
					   --GetSpaces((string.len(L["Places"]) - string.len(tostring(dm_herbs[idx].aLoot))))..
					   GetHexColoredText(format("%.2f",dm_buff[idx].aCount / dm_buff[idx].aLoot), COL_ORANGE).."\n"
		end
		TitanPlugins["DM"].tooltipTitle = L["Area"]..": "..search;
	end
			
	if (out == "") then
		out = L["No_corresponding_data_found"].."\n"
	end
	return header..out;
end

function TitanPanelDMButton_GetButtonText()
	return Addon
end

function TitanPanelDMButton_OnLoad(self)
	self:SetFrameStrata("FULLSCREEN");
	self.registry = {
		id = "DM",
		menuText = "Titan|c0070DE5f "..Addon.."|r",
		buttonTextFunction = "TitanPanelDMButton_GetButtonText",
		tooltipTitle = Addon..TitanUtils_GetGreenText("\r".."by Linuz@Proudmoore.Eu"),
		tooltipTextFunction = "TitanPanelDMButton_GetTooltipText",
		frequency = 1,
		icon = "Interface\\Icons\\Trade_herbalism.blp",
		iconWidth = 16,
		category = "Profession",
		controlVariables = {
			ShowIcon = true,
			ShowLabelText = true,
			ShowRegularText = false,
			ShowColoredText = true,
			DisplayOnRightSide = false
		},
		savedVariables = {
			ShowIcon = 1,
			ShowLabelText = 1,
			ShowColoredText = 1,
			logg = 1,
			logsize = 1,
		}
	}
	
	--Enable Slash Commands
	SlashCmdList["DataMiner"] = function(msg)
		DM_SlashHandler(string.lower(msg));
		end
	SLASH_DataMiner1 = "/dm"
	SLASH_DataMiner2 = "/dataminer"
	
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
end
